@@ -5,8 +5,10 @@ from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker
 
 Base = declarative_base()
+
+
 class CasbinRule(Base):
-    __tablename__ = 'casbin_rule'
+    __tablename__ = "casbin_rule"
 
     id = Column(Integer, primary_key=True)
     ptype = Column(String(255))
@@ -23,7 +25,7 @@ class CasbinRule(Base):
             if v is None:
                 break
             arr.append(v)
-        return ', '.join(arr)
+        return ", ".join(arr)
 
     def __repr__(self):
         return '<CasbinRule {}: "{}">'.format(self.id, str(self))
@@ -38,6 +40,7 @@ class Filter:
     v4 = []
     v5 = []
 
+
 class Adapter(persist.Adapter):
     """the interface for Casbin adapters."""
 
@@ -69,38 +72,31 @@ class Adapter(persist.Adapter):
     def load_filtered_policy(self, model, filter) -> None:
         """loads all policy rules from the storage."""
         query = self._session.query(self._db_class)
-        filters = self.filter_query(query,filter)
+        filters = self.filter_query(query, filter)
         filters = filters.all()
 
         for line in filters:
             persist.load_policy_line(str(line), model)
         self._filtered = True
-        
-    def filter_query(self,querydb,filter):
-        ret = []
-        if len(filter.ptype) >0:
-            ret = querydb.filter(CasbinRule.ptype.in_(filter.ptype)).order_by(CasbinRule.id)
-            return ret
-        if len(filter.v0) >0:
-            ret = querydb.filter(CasbinRule.v0.in_(filter.v0)).order_by(CasbinRule.id)
-            return ret
-        if len(filter.v1) >0:
-            ret = querydb.filter(CasbinRule.v1.in_(filter.v1)).order_by(CasbinRule.id)
-            return ret
-        if len(filter.v2) >0:
-            ret = querydb.filter(CasbinRule.v2.in_(filter.v2)).order_by(CasbinRule.id)
-            return ret
-        if len(filter.v3) >0:
-            ret = querydb.filter(CasbinRule.v3.in_(filter.v3)).order_by(CasbinRule.id)
-            return ret
-        if len(filter.v4) >0:
-            ret = querydb.filter(CasbinRule.v4.in_(filter.v4)).order_by(CasbinRule.id)
-            return ret
-        if len(filter.v5) >0:
-            ret = querydb.filter(CasbinRule.v5.in_(filter.v5)).order_by(CasbinRule.id)
-            return ret
 
-    def _save_policy_line(self,ptype,rule):
+    def filter_query(self, querydb, filter):
+        if len(filter.ptype) > 0:
+            querydb = querydb.filter(CasbinRule.ptype.in_(filter.ptype))
+        if len(filter.v0) > 0:
+            querydb = querydb.filter(CasbinRule.v0.in_(filter.v0))
+        if len(filter.v1) > 0:
+            querydb = querydb.filter(CasbinRule.v1.in_(filter.v1))
+        if len(filter.v2) > 0:
+            querydb = querydb.filter(CasbinRule.v2.in_(filter.v2))
+        if len(filter.v3) > 0:
+            querydb = querydb.filter(CasbinRule.v3.in_(filter.v3))
+        if len(filter.v4) > 0:
+            querydb = querydb.filter(CasbinRule.v4.in_(filter.v4))
+        if len(filter.v5) > 0:
+            querydb = querydb.filter(CasbinRule.v5.in_(filter.v5))
+        return querydb.order_by(CasbinRule.id)
+
+    def _save_policy_line(self, ptype, rule):
         line = self._db_class(ptype=ptype)
         for i, v in enumerate(rule):
             setattr(line, "v{}".format(i), v)
@@ -127,6 +123,12 @@ class Adapter(persist.Adapter):
         self._save_policy_line(ptype, rule)
         self._commit()
 
+    def add_policies(self, sec, ptype, rules):
+        """adds a policy rules to the storage."""
+        for rule in rules:
+            self._save_policy_line(ptype, rule)
+        self._commit()
+
     def remove_policy(self, sec, ptype, rule):
         """removes a policy rule from the storage."""
         query = self._session.query(self._db_class)
@@ -138,6 +140,16 @@ class Adapter(persist.Adapter):
 
         return True if r > 0 else False
 
+    def remove_policies(self, sec, ptype, rules):
+        """removes a policy rules from the storage."""
+        query = self._session.query(self._db_class)
+        query = query.filter(self._db_class.ptype == ptype)
+        for rule in rules:
+            query = query.filter(or_(getattr(self._db_class, "v{}".format(i)) == v for i, v in enumerate(rule)))
+        query.delete()
+        self._commit()
+
+
     def remove_filtered_policy(self, sec, ptype, field_index, *field_values):
         """removes policy rules that match the filter from the storage.
         This is part of the Auto-Save feature.
@@ -149,11 +161,12 @@ class Adapter(persist.Adapter):
         if not (1 <= field_index + len(field_values) <= 6):
             return False
         for i, v in enumerate(field_values):
-            query = query.filter(getattr(self._db_class, "v{}".format(field_index + i)) == v)
+            if v != '':
+                query = query.filter(getattr(self._db_class, "v{}".format(field_index + i)) == v)
         r = query.delete()
         self._commit()
 
         return True if r > 0 else False
 
     def __del__(self):
-        self._session.close()
\ No newline at end of file
+        self._session.close()
